from enum import Enum


class OCRLanguage(str, Enum):
    """
    Enum representing supported OCR languages.
    """
    DEFAULT = 'ChinesePRC,English,Digits'
    DIGITS = 'Digits'
    ENGLISH = 'English'
    CHINESE_PRC = 'ChinesePRC'
    CHINESE_TAIWAN = 'ChineseTaiwan'
    FRENCH = 'French'
    GERMAN = 'German'
    JAPANESE = 'Japanese'
    PORTUGUESE = 'Portuguese'
    SPANISH = 'Spanish'
    ITALIAN = 'Italian'
    BULGARIAN = 'Bulgarian'
    CROATIAN = 'Croatian'
    CZECH = 'Czech'
    DANISH = 'Danish'
    DUTCH = 'Dutch'
    FINNISH = 'Finnish'
    GREEK = 'Greek'
    HUNGARIAN = 'Hungarian'
    KOREAN = 'Korean'
    NORWEGIAN = 'Norwegian'
    POLISH = 'Polish'
    RUSSIAN = 'Russian'
    SLOVENIAN = 'Slovenian'
    SWEDISH = 'Swedish'
    TURKISH = 'Turkish'


class OCRFormat(str, Enum):
    """
    Enum representing supported OCR output formats.
    """
    PDF = 'pdf'
    DOCX = 'docx'
    PPTX = 'pptx'
    XLSX = 'xlsx'
    TXT = 'txt'


class T2IQuality(int, Enum):
    """
    Enum representing text-to-image generation quality levels.
    """
    LOW = 0
    HIGH = 1


class T2ITheme(int, Enum):
    """
    Enum representing themes for text-to-image generation.
    """
    GENERAL = 0
    DIGITAL_ART = 1
    _3D = 2
    PHOTOGRAPHY = 3
    ANIME = 4
    CYBERPUNK = 5
    PAINTING = 6
    PIXEL_ART = 7
    ILLUSTRATION = 8
    SKETCH = 9
    _UNKNOWN = 10


class T2ISize(tuple, Enum):
    """
    Enum representing supported image sizes for text-to-image generation.
    HD = High Definition
    FHD = Full High Definition
    """
    HD_1_1 = (616, 616)
    HD_3_2 = (768, 512)
    HD_4_3 = (704, 528)
    HD_3_4 = (528, 704)
    HD_16_9 = (904, 512)
    HD_9_16 = (512, 904)
    FHD_1_1 = (1024, 1024)
    FHD_3_2 = (1008, 672)
    FHD_4_3 = (1024, 768)
    FHD_3_4 = (768, 1024)
    FHD_16_9 = (1024, 576)
    FHD_9_16 = (576, 1024)



from dataclasses import dataclass, field
from pathlib import Path

from httpx import AsyncClient

from .enums import OCRFormat


@dataclass
class BaseImage:
    """
    Base class for processed images.
    """
    _http: AsyncClient
    url: str
    _cached_bytes: bytes | None = field(default=None, init=False, repr=False)

    async def get_bytes(self) -> bytes:
        """
        Fetches the image bytes from the URL.

        :return: The content of the image in bytes.
        :rtype: bytes
        """
        if self._cached_bytes is None:
            response = await self._http.get(self.url)
            self._cached_bytes = response.content
        return self._cached_bytes

    async def download(self, output: str) -> None:
        """
        Downloads the image and saves it to the specified file path.

        :param output: The file path where the image will be saved.
        :type output: str
        """
        Path(output).write_bytes(await self.get_bytes())


@dataclass
class EnhancedImage(BaseImage):
    """
    Represents an enhanced image.

    :ivar url: The URL of the enhanced image.
    :type url: str
    :ivar watermark: Indicates whether the image has a watermark.
    :type watermark: bool
    :param face_enhanced: Indicates whether the image has been enhanced for faces.
    :type face_enhanced: bool
    """
    watermark: bool
    face_enhanced: bool


@dataclass
class BackgroundRemovedImage(BaseImage):
    """
    Represents an enhanced image.

    :ivar url: The URL of the enhanced image.
    :type url: str
    :ivar watermark: Indicates whether the image has a watermark.
    :type watermark: bool
    :param mask: The URL of the mask used for background removal.
    :type mask: str
    """
    watermark: bool
    mask: str


@dataclass
class OCRResult(BaseImage):
    """
    Represents an OCR result.

    :ivar url: The URL used to get OCR result.
    :type url: str
    :ivar format: The format of the OCR result.
    :type format: OCRFormat
    """
    format: OCRFormat

    async def text(self, encoding: str = 'utf-8', errors: str = 'ignore') -> str:
        """
        Returns the OCR result as string.
        """
        bytes_ = await self.get_bytes()
        return bytes_.decode(encoding=encoding, errors=errors)


@dataclass
class T2IResult(BaseImage):
    """
    Represents the result of a text-to-image.

    :ivar url: The URL used to get T2I result.
    :type url: str
    :ivar id: The ID of the generated image.
    :type id: str
    """
    id: str


@dataclass
class ColorizeResult(BaseImage):
    """
    Represents the result of a colorization operation.

    :ivar url: The URL used to retrieve the colorization result.
    :type url: str
    """



"""
PicWish Python
==============
A library designed to provide free and unlimited access to PicWish
services such as AI image generation, image enhancement, image OCR,
and background removal.

Copyright (c) d60 2024

This software is released under the MIT License.
For more details, see https://opensource.org/licenses/MIT.
"""

__version__ = '0.4.7'
__license__ = 'MIT'

import asyncio
import os

from .enums import OCRFormat, OCRLanguage, T2IQuality, T2ISize, T2ITheme
from .image_models import BackgroundRemovedImage, ColorizeResult, EnhancedImage, OCRResult, T2IResult
from .main import EnhancedImage, PicWish, PicwishError

if os.name == 'nt':
    asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
